-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Reusable test bench stimuli to setup tech_mac_10g via MM
-- Description:

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE work.tb_tech_mac_10g_pkg.ALL;
USE work.tech_mac_10g_component_pkg.ALL;


ENTITY tb_tech_mac_10g_setup IS
  GENERIC (
    g_technology    : NATURAL := c_tech_select_default;
    g_nof_macs      : POSITIVE := 1;
    g_src_mac       : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"123456789ABC"  -- = 12-34-56-78-9A-BC
  );
  PORT (
    tb_end    : IN  STD_LOGIC := '0';
    mm_clk    : OUT STD_LOGIC;
    mm_rst    : OUT STD_LOGIC;
    mm_init   : OUT STD_LOGIC;
    mac_mosi  : OUT t_mem_mosi;
    mac_miso  : IN  t_mem_miso
  );
END tb_tech_mac_10g_setup;


ARCHITECTURE tb OF tb_tech_mac_10g_setup IS

  CONSTANT mm_clk_period        : TIME := 20 ns;    --  50 MHz

  -- Clocks and reset
  SIGNAL i_mm_clk          : STD_LOGIC := '0';  -- memory-mapped bus clock
  SIGNAL i_mm_rst          : STD_LOGIC;         -- reset synchronous with mm_clk
    
BEGIN

  mm_clk <= i_mm_clk;
  mm_rst <= i_mm_rst;
  
  i_mm_clk <= NOT i_mm_clk OR tb_end AFTER mm_clk_period/2;   -- MM clock  
  i_mm_rst <= '1', '0' AFTER mm_clk_period*10;

  p_mm_setup : PROCESS
    VARIABLE v_mm_offset : NATURAL;
  BEGIN
    mm_init  <= '1';
    mac_mosi.wr <= '0';
    mac_mosi.rd <= '0';

    -- wait until after reset release
    proc_common_wait_until_low(i_mm_clk, i_mm_rst);
    proc_common_wait_some_cycles(i_mm_clk, 10);

    FOR I IN 0 TO g_nof_macs-1 LOOP
      v_mm_offset := I * 2**func_tech_mac_10g_csr_addr_w(g_technology);
      proc_tech_mac_10g_setup(g_technology,
                              g_src_mac,
                              v_mm_offset,
                              i_mm_clk, mac_miso, mac_mosi);
    END LOOP;
    mm_init <= '0';
    WAIT;
  END PROCESS;

END tb;
